﻿using SushiBarBusinessLogic.OfficePackage.HelperModels;
using SushiBarBusinessLogic.OfficePackage;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;

namespace SushiBarBusinessLogic.BusinessLogics
{
    public class ReportLogic : IReportLogic
    {
        private readonly IIngredientStorage _ingredientStorage;

        private readonly ISushiStorage _sushiStorage;

        private readonly IOrderStorage _orderStorage;

        private readonly AbstractSaveToExcel _saveToExcel;

        private readonly AbstractSaveToWord _saveToWord;

        private readonly AbstractSaveToPdf _saveToPdf;

        public ReportLogic(ISushiStorage productStorage, IIngredientStorage ingredientStorage, IOrderStorage orderStorage,
            AbstractSaveToExcel saveToExcel, AbstractSaveToWord saveToWord, AbstractSaveToPdf saveToPdf)
        {
            _sushiStorage = productStorage;
            _ingredientStorage = ingredientStorage;
            _orderStorage = orderStorage;

            _saveToExcel = saveToExcel;
            _saveToWord = saveToWord;
            _saveToPdf = saveToPdf;
        }

        /// <summary>
        /// Получение списка ингредиентов с указанием, в каких суши используются
        /// </summary>
        /// <returns></returns>
        public List<ReportSushiIngredientViewModel> GetSushiIngredient()
        {
            var ingredients = _ingredientStorage.GetFullList();

            var sushiList = _sushiStorage.GetFullList();

            var list = new List<ReportSushiIngredientViewModel>();

            foreach (var sushi in sushiList)
            {
                var record = new ReportSushiIngredientViewModel
                {
                    SushiName = sushi.SushiName,
                    Ingredients = new(),
                    TotalCount = 0
                };
                foreach (var ingredient in ingredients)
                {
                    if (sushi.SushiIngredients.ContainsKey(ingredient.Id))
                    {
                        record.Ingredients.Add(new(ingredient.IngredientName, 
                            sushi.SushiIngredients[ingredient.Id].Item2));
                        record.TotalCount += sushi.SushiIngredients[ingredient.Id].Item2;
                    }
                }

                list.Add(record);
            }

            return list;
        }

        /// <summary>
        /// Получение списка заказов за определенный период
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ReportOrdersViewModel> GetOrders(ReportBindingModel model)
        {
            return _orderStorage.GetFilteredList(new OrderSearchModel { DateFrom = model.DateFrom, DateTo = model.DateTo })
                    .Select(x => new ReportOrdersViewModel
                    {
                        Id = x.Id,
                        DateCreate = x.DateCreate,
                        SushiName = x.SushiName,
                        OrderStatus = x.Status.ToString(),
                        Sum = x.Sum
                    })
                    .ToList();
        }

        /// <summary>
        /// Сохранение суши в файл-Word
        /// </summary>
        /// <param name="model"></param>
        public void SaveListSushiToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateDoc(new WordInfo
            {
                FileName = model.FileName,
                Title = "Список суши",
                ListSushi = _sushiStorage.GetFullList()
            });
        }

        /// <summary>
        /// Сохранение ингредиентов с указаеним суши в файл-Excel
        /// </summary>
        /// <param name="model"></param>
        public void SaveSushiIngredientToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateReport(new ExcelInfo
            {
                FileName = model.FileName,
                Title = "Список суши",
                SushiIngredients = GetSushiIngredient()
            });
        }

        /// <summary>
        /// Сохранение заказов в файл-Pdf
        /// </summary>
        /// <param name="model"></param>
        public void SaveOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateDoc(new PdfInfo
            {
                FileName = model.FileName,
                Title = "Список заказов",
                DateFrom = model.DateFrom!.Value,
                DateTo = model.DateTo!.Value,
                Orders = GetOrders(model)
            });
        }
    }
}
